@@ -38,7 +38,7 @@ GEVulkanTexture::GEVulkanTexture(const std::string& path,
                  m_internal_format(VK_FORMAT_R8G8B8A8_UNORM),
                  m_vk(getVKDriver())
 {
-    m_max_size = getDriver()->getDriverAttributes()
+    core::dimension2du max_size = getDriver()->getDriverAttributes()
         .getAttributeAsDimension2d("MAX_TEXTURE_SIZE");
     m_full_path = getDriver()->getFileSystem()->getAbsolutePath(NamedPath);
     if (!getDriver()->getFileSystem()->existFileOnly(m_full_path))
@@ -48,15 +48,17 @@ GEVulkanTexture::GEVulkanTexture(const std::string& path,
     }
 
     auto& paths = getGEConfig()->m_ondemand_load_texture_paths;
-    m_ondemand_load = (paths.find(m_full_path.c_str()) != paths.end());
+    auto path_itr = paths.find(m_full_path.c_str());
+    m_ondemand_load = (path_itr != paths.end());
     if (m_ondemand_load)
     {
+        paths.erase(path_itr);
         video::IImageLoader* loader = NULL;
         io::IReadFile* file = io::createReadFile(m_full_path);
         getDriver()->createImageFromFile(file, &loader);
         if (loader && loader->getImageSize(file, &m_orig_size))
         {
-            m_size = getResizingTarget(m_orig_size, m_max_size);
+            m_size = getResizingTarget(m_orig_size, max_size);
             if (m_size.Width < 4 || m_size.Height < 4)
                 m_has_mipmaps = false;
             setPlaceHolderView();
@@ -71,7 +73,7 @@ GEVulkanTexture::GEVulkanTexture(const std::string& path,
     m_image_view_lock.lock();
     m_thread_loading_lock.lock();
     GEVulkanCommandLoader::addMultiThreadingCommand(
-        std::bind(&GEVulkanTexture::reloadInternal, this));
+        std::bind(&GEVulkanTexture::reloadInternal, this, max_size));
 }   // GEVulkanTexture
 
 // ----------------------------------------------------------------------------
@@ -289,10 +291,14 @@ bool GEVulkanTexture::createImage(VkImageUsageFlags usage)
     if (m_image_view_type == VK_IMAGE_VIEW_TYPE_CUBE ||
         m_image_view_type == VK_IMAGE_VIEW_TYPE_CUBE_ARRAY)
         image_info.flags = VK_IMAGE_CREATE_CUBE_COMPATIBLE_BIT;
+    if (m_internal_format != getSRGBformat(m_internal_format))
+        image_info.flags |= VK_IMAGE_CREATE_MUTABLE_FORMAT_BIT;
 
     m_vma_info = {};
     VmaAllocationCreateInfo alloc_info = {};
     alloc_info.usage = VMA_MEMORY_USAGE_AUTO_PREFER_DEVICE;
+    if ((usage & VK_IMAGE_USAGE_TRANSIENT_ATTACHMENT_BIT) != 0)
+        alloc_info.preferredFlags = VK_MEMORY_PROPERTY_LAZILY_ALLOCATED_BIT;
     VkResult result = vmaCreateImage(m_vk->getVmaAllocator(), &image_info,
         &alloc_info, &m_image, &m_vma_allocation, &m_vma_info);
 
@@ -386,6 +392,33 @@ void GEVulkanTexture::transitionImageLayout(VkCommandBuffer command_buffer,
         source_stage = VK_PIPELINE_STAGE_TOP_OF_PIPE_BIT;
         destination_stage = VK_PIPELINE_STAGE_EARLY_FRAGMENT_TESTS_BIT;
     }
+    else if (old_layout == VK_IMAGE_LAYOUT_UNDEFINED &&
+        new_layout == VK_IMAGE_LAYOUT_GENERAL)
+    {
+        barrier.srcAccessMask = 0;
+        barrier.dstAccessMask = VK_ACCESS_SHADER_READ_BIT;
+
+        source_stage = VK_PIPELINE_STAGE_TOP_OF_PIPE_BIT;
+        destination_stage = VK_PIPELINE_STAGE_FRAGMENT_SHADER_BIT;
+    }
+    else if (old_layout == VK_IMAGE_LAYOUT_UNDEFINED &&
+        new_layout == VK_IMAGE_LAYOUT_SHADER_READ_ONLY_OPTIMAL)
+    {
+        barrier.srcAccessMask = 0;
+        barrier.dstAccessMask = VK_ACCESS_SHADER_READ_BIT;
+
+        source_stage = VK_PIPELINE_STAGE_TOP_OF_PIPE_BIT;
+        destination_stage = VK_PIPELINE_STAGE_FRAGMENT_SHADER_BIT;
+    }
+    else if (old_layout == VK_IMAGE_LAYOUT_TRANSFER_DST_OPTIMAL &&
+        new_layout == VK_IMAGE_LAYOUT_GENERAL)
+    {
+        barrier.srcAccessMask = VK_ACCESS_TRANSFER_WRITE_BIT;
+        barrier.dstAccessMask = VK_ACCESS_SHADER_READ_BIT;
+
+        source_stage = VK_PIPELINE_STAGE_TRANSFER_BIT;
+        destination_stage = VK_PIPELINE_STAGE_FRAGMENT_SHADER_BIT;
+    }
     else
     {
         return;
@@ -417,7 +450,8 @@ void GEVulkanTexture::copyBufferToImage(VkCommandBuffer command_buffer,
 }   // copyBufferToImage
 
 // ----------------------------------------------------------------------------
-bool GEVulkanTexture::createImageView(VkImageAspectFlags aspect_flags)
+bool GEVulkanTexture::createImageView(VkImageAspectFlags aspect_flags,
+                                      bool create_srgb_view)
 {
     VkImageViewCreateInfo view_info = {};
     view_info.sType = VK_STRUCTURE_TYPE_IMAGE_VIEW_CREATE_INFO;
@@ -445,6 +479,19 @@ bool GEVulkanTexture::createImageView(VkImageAspectFlags aspect_flags)
     {
         image_view.get()->store(view_ptr);
         m_image_view = image_view;
+        VkFormat srgb_format = getSRGBformat(m_internal_format);
+        if (create_srgb_view && m_internal_format != srgb_format)
+        {
+            image_view = std::make_shared<std::atomic<VkImageView> >();
+            view_info.format = srgb_format;
+            view_ptr = VK_NULL_HANDLE;
+            if (vkCreateImageView(m_vulkan_device, &view_info,
+                NULL, &view_ptr) == VK_SUCCESS)
+            {
+                image_view.get()->store(view_ptr);
+                m_image_view_srgb = image_view;
+            }
+        }
 
         if (m_placeholder_view)
             m_placeholder_view.get()->store(VK_NULL_HANDLE);
@@ -466,6 +513,13 @@ void GEVulkanTexture::clearVulkanData()
         vkDestroyImageView(m_vulkan_device, m_image_view.get()->load(), NULL);
         m_image_view.get()->store(VK_NULL_HANDLE);
         m_image_view.reset();
+        if (m_image_view_srgb)
+        {
+            vkDestroyImageView(m_vulkan_device,
+                m_image_view_srgb.get()->load(), NULL);
+            m_image_view_srgb.get()->store(VK_NULL_HANDLE);
+            m_image_view_srgb.reset();
+        }
     }
     if (m_image != VK_NULL_HANDLE)
     {
@@ -477,19 +531,18 @@ void GEVulkanTexture::clearVulkanData()
 }   // clearVulkanData
 
 // ----------------------------------------------------------------------------
-void GEVulkanTexture::reloadInternal()
+void GEVulkanTexture::reloadInternal(const core::dimension2du& max_size)
 {
     if (m_disable_reload)
         return;
 
     clearVulkanData();
 
-    video::IImage* texture_image = NULL;
-    if (m_ondemand_load)
+    video::IImage* texture_image = getResizedImageFullPath(m_full_path,
+        max_size, &m_orig_size);
+    if (texture_image == NULL)
     {
-        texture_image = getResizedImageFullPath(m_full_path, m_max_size,
-            NULL, &m_size);
-        if (texture_image == NULL)
+        if (m_ondemand_load)
         {
             printf("Missing texture_image in getResizedImageFullPath when "
                 "reloadInternal during ondemand loading for %s\n",
@@ -499,20 +552,18 @@ void GEVulkanTexture::reloadInternal()
             m_thread_loading_lock.unlock();
             return;
         }
-    }
-    else
-    {
-        texture_image = getResizedImageFullPath(m_full_path, m_max_size,
-            &m_orig_size);
-        if (texture_image == NULL)
+        else
         {
             throw std::runtime_error(
                 "Missing texture_image in getResizedImageFullPath");
         }
-        m_size = texture_image->getDimension();
-        if (m_size.Width < 4 || m_size.Height < 4)
-            m_has_mipmaps = false;
     }
+
+    m_size = texture_image->getDimension();
+    if (m_size.Width < 4 || m_size.Height < 4)
+        m_has_mipmaps = false;
+    else
+        m_has_mipmaps = true;
     m_size_lock.unlock();
 
     if (m_image_mani)
@@ -574,8 +625,10 @@ uint8_t* GEVulkanTexture::getTextureData()
         if (m_full_path.empty())
             return NULL;
 
+        const core::dimension2du& max_size = getDriver()->getDriverAttributes()
+            .getAttributeAsDimension2d("MAX_TEXTURE_SIZE");
         video::IImage* texture_image = getResizedImageFullPath(m_full_path,
-            m_max_size, NULL, &m_size);
+            max_size, NULL, &m_size);
         if (texture_image == NULL)
             return NULL;
         texture_image->setDeleteMemory(false);
@@ -850,11 +903,13 @@ void GEVulkanTexture::reload()
     }
     else if (!m_disable_reload)
     {
+        core::dimension2du max_size = getDriver()->getDriverAttributes()
+            .getAttributeAsDimension2d("MAX_TEXTURE_SIZE");
         m_size_lock.lock();
         m_image_view_lock.lock();
         m_thread_loading_lock.lock();
         GEVulkanCommandLoader::addMultiThreadingCommand(
-            std::bind(&GEVulkanTexture::reloadInternal, this));
+            std::bind(&GEVulkanTexture::reloadInternal, this, max_size));
     }
 }   // reload
 
@@ -870,20 +925,28 @@ void GEVulkanTexture::setPlaceHolderView()
 }   // setPlaceHolderView
 
 //-----------------------------------------------------------------------------
-std::shared_ptr<std::atomic<VkImageView> > GEVulkanTexture::getImageViewLive() const
+std::shared_ptr<std::atomic<VkImageView> > GEVulkanTexture::getImageViewLive(
+                                                               bool srgb) const
 {
     assert(m_ondemand_load && m_placeholder_view);
     if (m_ondemand_loading.load() == false)
     {
         if (m_image_view)
-            return m_image_view;
+        {
+            if (srgb && m_image_view_srgb)
+                return m_image_view_srgb;
+            else
+                return m_image_view;
+        }
         else
         {
             GEVulkanTexture* tex = const_cast<GEVulkanTexture*>(this);
+            core::dimension2du max_size = getDriver()->getDriverAttributes()
+                .getAttributeAsDimension2d("MAX_TEXTURE_SIZE");
             tex->m_thread_loading_lock.lock();
             tex->m_ondemand_loading.store(true);
             GEVulkanCommandLoader::addMultiThreadingCommand(
-                std::bind(&GEVulkanTexture::reloadInternal, tex));
+                std::bind(&GEVulkanTexture::reloadInternal, tex, max_size));
             return m_placeholder_view;
         }
     }
